#*******************************************************************************
# Copyright (C) 2007 Novell, Inc. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of Novell, Inc. nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL Novell, Inc. OR THE CONTRIBUTORS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
#*****************************************************************************/
"""Python Provider for OMC_AccountManagementCapabilities

Instruments the CIM class OMC_AccountManagementCapabilities

"""

import pywbem
import pwd

class OMC_AccountManagementCapabilitiesProvider(pywbem.CIMProvider):
    """Instrument the CIM class OMC_AccountManagementCapabilities 

    AccountManagementCapabilities describes the capabilities supported for
    managing Accounts associated with an instance of
    AccountManagementService. AccountManagementCapabilities is associated
    with an instance of AccountManagementService through the
    ElementCapabilities association.
    
    """

    def __init__ (self, env):
        logger = env.get_logger()
        logger.log_debug('Initializing provider %s from %s' \
                % (self.__class__.__name__, __file__))
        # If you will be filtering instances yourself according to 
        # property_list, role, result_role, and result_class_name 
        # parameters, set self.filter_results to False
        # self.filter_results = False

    def get_instance(self, env, model, cim_class):
        """Return an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstance to be returned.  The 
            key properties are set on this instance to correspond to the 
            instanceName that was requested.  The properties of the model
            are already filtered according to the PropertyList from the 
            request.  Only properties present in the model need to be
            given values.  If you prefer, you can set all of the 
            values, and the instance will be filtered for you. 
        cim_class -- The pywbem.CIMClass

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized 
            or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM 
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        
        #model['Caption'] = # TODO (type = unicode) 
        #model['Description'] = # TODO (type = unicode) 
        #model['ElementName'] = # TODO (type = unicode) (Required)
        #model['ElementNameEditSupported'] = # TODO (type = bool) 
        #model['ElementNameMask'] = # TODO (type = unicode) 
        #model['MaxElementNameLen'] = # TODO (type = pywbem.Uint16) 
        #model['OperationsSupported'] = # TODO (type = [pywbem.Uint16,] self.Values.OperationsSupported) 
        model['ElementName'] = 'AccountManagementCapability'
        model['ElementNameEditSupported'] = False     
        return model

    def enum_instances(self, env, model, cim_class, keys_only):
        """Enumerate instances.

        The WBEM operations EnumerateInstances and EnumerateInstanceNames
        are both mapped to this method. 
        This method is a python generator

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstances to be generated.  
            The properties of the model are already filtered according to 
            the PropertyList from the request.  Only properties present in 
            the model need to be given values.  If you prefer, you can 
            always set all of the values, and the instance will be filtered 
            for you. 
        cim_class -- The pywbem.CIMClass
        keys_only -- A boolean.  True if only the key properties should be
            set on the generated instances.

        Possible Errors:
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)

        #while true: # TODO more instances?
            # TODO fetch system resource
            # Key properties    
        model['InstanceID'] = 'OMC:AccountCapability'
        if keys_only:
            yield model
        else:
            try:
                yield self.get_instance(env, model, cim_class)
            except pywbem.CIMError, (num, msg):
                if num not in (pywbem.CIM_ERR_NOT_FOUND, 
                               pywbem.CIM_ERR_ACCESS_DENIED):
                    raise

    def set_instance(self, env, instance, previous_instance, cim_class):
        """Return a newly created or modified instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance -- The new pywbem.CIMInstance.  If modifying an existing 
            instance, the properties on this instance have been filtered by 
            the PropertyList from the request.
        previous_instance -- The previous pywbem.CIMInstance if modifying 
            an existing instance.  None if creating a new instance. 
        cim_class -- The pywbem.CIMClass

        Return the new instance.  The keys must be set on the new instance. 

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized 
            or otherwise incorrect parameters)
        CIM_ERR_ALREADY_EXISTS (the CIM Instance already exists -- only 
            valid if previous_instance is None, indicating that the operation
            was CreateInstance)
        CIM_ERR_NOT_FOUND (the CIM Instance does not exist -- only valid 
            if previous_instance is not None, indicating that the operation
            was ModifyInstance)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        #logger = env.get_logger()
        #logger.log_debug('Entering %s.set_instance()' \
        #       % self.__class__.__name__)
        # TODO create or modify the instance
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        #return instance

    def delete_instance(self, env, instance_name):
        """Delete an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance_name -- A pywbem.CIMInstanceName specifying the instance 
            to delete.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_NAMESPACE
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized 
            or otherwise incorrect parameters)
        CIM_ERR_INVALID_CLASS (the CIM Class does not exist in the specified 
            namespace)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM 
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """ 

        #logger = env.get_logger()
        #logger.log_debug('Entering %s.delete_instance()' \
        #       % self.__class__.__name__)

        # TODO delete the resource
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        
    def cim_method_creategoalsettings(self, env, object_name, method,
                                      param_supportedgoalsettings,
                                      param_templategoalsettings):
        """Implements OMC_AccountManagementCapabilities.CreateGoalSettings()

        Method to create a set of supported SettingData elements, from two
        sets of SettingData elements, provided by the caller.  CreateGoal
        should be used when the SettingData instances that represents the
        goal will not persist beyond the execution of the client and where
        those instances are not intended to be shared with other,
        non-cooperating clients.  Both TemplateGoalSettings and
        SupportedGoalSettings are represented as strings containing
        EmbeddedInstances of a CIM_SettingData subclass. These embedded
        instances do not exist in the infrastructure supporting this
        method but are maintained by the caller/client.  This method
        should return CIM_Error(s) representing that a single named
        property of a setting (or other) parameter (either reference or
        embedded object) has an invalid value or that an invalid
        combination of named properties of a setting (or other) parameter
        (either reference or embedded object) has been requested.  If the
        input TemplateGoalSettings is NULL or the empty string, this
        method returns a default SettingData element that is supported by
        this Capabilities element.  If the TemplateGoalSettings specifies
        values that cannot be supported, this method shall return an
        appropriate CIM_Error and should return a best match for a
        SupportedGoalSettings.  The client proposes a goal using the
        TemplateGoalSettings parameter and gets back Success if the
        TemplateGoalSettings is exactly supportable. It gets back
        "Alternative Proposed" if the output SupportedGoalSettings
        represents a supported alternative. This alternative should be a
        best match, as defined by the implementation.  If the
        implementation is conformant to a RegisteredProfile, then that
        profile may specify the algorithms used to determine best match. A
        client may compare the returned value of each property against the
        requested value to determine if it is left unchanged, degraded or
        upgraded.   Otherwise, if the TemplateGoalSettings is not
        applicable an "Invalid Parameter" error is returned.   When a
        mutually acceptable SupportedGoalSettings has been achieved, the
        client may use the contained SettingData instances as input to
        methods for creating a new object ormodifying an existing object.
        Also the embedded SettingData instances returned in the
        SupportedGoalSettings may be instantiated via CreateInstance,
        either by a client or as a side-effect of the execution of an
        extrinsic method for which the returned SupportedGoalSettings is
        passed as an embedded instance.
        
        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName 
            specifying the object on which the method CreateGoalSettings() 
            should be invoked.
        method -- A pywbem.CIMMethod representing the method meta-data
        param_supportedgoalsettings --  The input parameter SupportedGoalSettings (type pywbem.CIMInstance(classname='CIM_SettingData', ...)) 
            SupportedGoalSettings are elements of class CIM_SettingData, or
            a derived class.  At most, one instance of each SettingData
            subclass may be supplied.  All SettingData instances provided
            by this property are interpreted as a set, relative to this
            Capabilities instance.   To enable a client to provide
            additional information towards achieving the
            TemplateGoalSettings, an input set of SettingData instances
            may be provided. If not provided, this property shall be set
            to NULL on input.. Note that when provided, what property
            values are changed, and how, is implementation dependent and
            may be the subject of other standards.  If provided, the input
            SettingData instances must be ones that the implementation is
            able to support relative to the ManagedElement associated via
            ElementCapabilities. Typically, the input SettingData
            instances are created by a previous instantiation of
            CreateGoalSettings.  If the input SupportedGoalSettings is not
            supported by the implementation, then an "Invalid Parameter"
            (5) error is returned by this call. In this case, a
            corresponding CIM_ERROR should also be returned.  On output,
            this property is used to return the best supported match to
            the TemplateGoalSettings.  If the output SupportedGoalSettings
            matches the input SupportedGoalSettings, then the
            implementation is unable to improve further towards meeting
            the TemplateGoalSettings.
            
        param_templategoalsettings --  The input parameter TemplateGoalSettings (type pywbem.CIMInstance(classname='CIM_SettingData', ...)) 
            If provided, TemplateGoalSettings are elements of class
            CIM_SettingData, or a derived class, that is used as the
            template to be matched. .  At most, one instance of each
            SettingData subclass may be supplied.  All SettingData
            instances provided by this property are interpreted as a set,
            relative to this Capabilities instance.  SettingData instances
            that are not relevant to this instance are ignored.  If not
            provided, it shall be set to NULL. In that case, a SettingData
            instance representing the default settings of the associated
            ManagedElement is used.
            

        Returns a two-tuple containing the return value (type pywbem.Uint16 self.Values.CreateGoalSettings)
        and a dictionary with the out-parameters

        Output parameters:
        SupportedGoalSettings -- (type pywbem.CIMInstance(classname='CIM_SettingData', ...)) 
            SupportedGoalSettings are elements of class CIM_SettingData, or
            a derived class.  At most, one instance of each SettingData
            subclass may be supplied.  All SettingData instances provided
            by this property are interpreted as a set, relative to this
            Capabilities instance.   To enable a client to provide
            additional information towards achieving the
            TemplateGoalSettings, an input set of SettingData instances
            may be provided. If not provided, this property shall be set
            to NULL on input.. Note that when provided, what property
            values are changed, and how, is implementation dependent and
            may be the subject of other standards.  If provided, the input
            SettingData instances must be ones that the implementation is
            able to support relative to the ManagedElement associated via
            ElementCapabilities. Typically, the input SettingData
            instances are created by a previous instantiation of
            CreateGoalSettings.  If the input SupportedGoalSettings is not
            supported by the implementation, then an "Invalid Parameter"
            (5) error is returned by this call. In this case, a
            corresponding CIM_ERROR should also be returned.  On output,
            this property is used to return the best supported match to
            the TemplateGoalSettings.  If the output SupportedGoalSettings
            matches the input SupportedGoalSettings, then the
            implementation is unable to improve further towards meeting
            the TemplateGoalSettings.
            

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, 
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not 
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor 
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_creategoalsettings()' \
                % self.__class__.__name__)

        # TODO do something
        raise pywbem.CIMError(pywbem.CIM_ERR_METHOD_NOT_AVAILABLE) # Remove to implemented
        out_params = {}
        #out_params['supportedgoalsettings'] = # TODO (type pywbem.CIMInstance(classname='CIM_SettingData', ...))
        rval = None # TODO (type pywbem.Uint16 self.Values.CreateGoalSettings)
        return (rval, out_params)
        
    class Values(object):
        class RequestedStatesSupported(object):
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Shut_Down = pywbem.Uint16(4)
            Offline = pywbem.Uint16(6)
            Test = pywbem.Uint16(7)
            Defer = pywbem.Uint16(8)
            Quiesce = pywbem.Uint16(9)
            Reboot = pywbem.Uint16(10)
            Reset = pywbem.Uint16(11)

        class CreateGoalSettings(object):
            Success = pywbem.Uint16(0)
            Not_Supported = pywbem.Uint16(1)
            Unknown = pywbem.Uint16(2)
            Timeout = pywbem.Uint16(3)
            Failed = pywbem.Uint16(4)
            Invalid_Parameter = pywbem.Uint16(5)
            Alternative_Proposed = pywbem.Uint16(6)
            # DMTF_Reserved = ..
            # Vendor_Specific = 32768..65535

        class OperationsSupported(object):
            Create = pywbem.Uint16(2)
            Modify = pywbem.Uint16(3)
            Delete = pywbem.Uint16(4)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 0x8000..0xFFFF

## end of class OMC_AccountManagementCapabilitiesProvider

def get_providers(env): 
    omc_accountmanagementcapabilities_prov = OMC_AccountManagementCapabilitiesProvider(env)  
    return {'OMC_AccountManagementCapabilities': omc_accountmanagementcapabilities_prov} 
